/*-----------------------------------------------------------------------------
FILENAME: bus.cpp
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 17.8.2008
VERSION: 1.0
LICENSE: GPL v3
DESCRIPTION: Implementation of Bus class (RS485 communication)
-----------------------------------------------------------------------------*/

#include "bus.h"
#include "global.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>

Bus bus;

Bus::Bus(void)
{
	RSDIRDDR |= 1<<RSDIRPINX;
	UCSRB = 0<<RXCIE | 1<<TXCIE | 1<<TXEN;
	UCSRC = 1<<URSEL | 1<<UCSZ1 | 1<<UCSZ0;
	UBRRH = 0;
	UBRRL = 47;
}

void Bus::Execute(void)
{
	
}

void Bus::OnTag(uint8_t *tag, bool accessed)
{
	sprintf(txbuf, "\x02R %.2X%.2X%.2X%.2X%.2X\x04", tag[0], tag[1], tag[2], tag[3], tag[4]);
	txbuf[2] = (accessed) ? '+' : '-';
	txbufpos = 0;
	txbuflen = strlen(txbuf);
	UCSRB |= 1<<UDRIE;
	RSDIRPORT |= 1<<RSDIRPINX;
}

void Bus::SendNextByte(void)
{
	UDR = txbuf[txbufpos++];
	if (txbufpos == txbuflen)
		UCSRB &= ~(1<<UDRIE);
}

ISR (SIG_UART_TRANS)
{
	RSDIRPORT &= ~(1<<RSDIRPINX);
}

ISR (SIG_UART_DATA)
{
	bus.SendNextByte();
}
