/*-----------------------------------------------------------------------------
FILENAME: button.cpp
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 17.8.2008
VERSION: 1.0
LICENSE: GPL v3
DESCRIPTION: Implementation of Button class
-----------------------------------------------------------------------------*/

#include "button.h"
#include "global.h"

#ifdef USEEXTEEPROM
#include "i2ceeprom.h"
#else
#include <avr/eeprom.h>
#endif

Button button;

Button::Button(void)
{
	BUTTONPORT |= 1<<BUTTONPINX; //enable pull up for button input
	laststate = 1<<BUTTONPINX;
}

void Button::Execute(void)
{
	uint8_t state;
	
	state = BUTTONPIN & (1<<BUTTONPINX);
	
	if (state != laststate)
	{
		laststate = state;
		if (!state)
			timer.SetTime(2000);
		else
			timer.SetTime(0);
	}
	
	if (timer.IsFlagged())
	{//Button was pushed longer than 2 s
		#ifdef LEDRED
		LEDRED;
		#endif
		#ifdef LEDON
		LEDON;
		#endif
		#ifdef USEEXTEEPROM
		i2ceeprom.Empty();
		#else
		uint16_t i;
		for (i=0; i<E2END; i++)
			eeprom_write_byte((uint8_t *) i, 0xFF);
		#endif
		LEDOFF;
	}
}
