/*-----------------------------------------------------------------------------
FILENAME: main.cpp
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 22.7.2008
VERSION: 1.0
LICENSE: GPL v3
DESCRIPTION:  Some hardware initialization and main loop
-----------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "task.h"
#include "timer.h"
#include "rfid.h"
#include "tagmanager.h"
#include "global.h"

int main(void)
{
	RELAISDDR |= 1<<RELAISPINX;
	
	#ifdef LEDADDR
	LEDADDR |= 1<<LEDAPINX;
	LEDBDDR |= 1<<LEDBPINX;
	#endif
	
	#ifdef LEDDDR
	LEDDDR |= 1<<LEDPINX;
	#endif
	
	#ifdef SHDDDR
	SHDDDR |= 1<<SHDPINX; //EM4095 shutdown pin
	#endif
	
	PORTB |= 1<<PB3; //enable pull up PB3 (MOSI) for jumper
	
	//initialize timer for Timer objects
	TCCR2 = 1<<WGM21 | 1<<CS22 | 1<<CS21;
	OCR2 = 29;
	TIMSK |= 1<<OCIE2;
	
	sei();
	
	tagmanager.Init();
	rfid.Init();
	
	while (1)
	{
		task_execute();
	}
}
