/*-----------------------------------------------------------------------------
FILENAME: tagmanager.cpp
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 22.7.2008
VERSION: 1.0
LICENSE: GPL v3
DESCRIPTION: Implementation of tagmanager class
-----------------------------------------------------------------------------*/

#include "tagmanager.h"
#include <string.h>
#include "global.h"
#include "bus.h"
#ifdef USEEXTEEPROM
#include "i2ceeprom.h"
#else
#include <avr/eeprom.h>
#endif

Tagmanager tagmanager;

const uint8_t empty_tag[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

Tagmanager::Tagmanager(void)
{
	mode = TMM_IDLE;
	admin = false;
}

void Tagmanager::Init(void)
{
}

void Tagmanager::Execute(void)
{
	uint8_t temp[5];
	
	switch (mode)
	{
		case TMM_LOOKUP:	
			#ifdef USEEXTEEPROM
			i2ceeprom.ReadBuf(temp, eeprom_ptr, 5);
			#else
			eeprom_read_block(temp, (uint8_t *) eeprom_ptr, 5);
			#endif
			
			if (memcmp(temp, tag, 5) == 0)
			{
				GiveAccess(); //Tag found
				if (eeprom_ptr == 0) //administrator tag ? (first tag in eeprom)
					admin = true;
			}
			else
				if (memcmp(temp, empty_tag, 5) == 0)
				{//eeprom tag is empty, we are at the end of eeprom-tags
					if (eeprom_ptr == 0)
					{//no tags in eeprom; add tag (this is now the administrator tag)
						#ifdef USEEXTEEPROM
						i2ceeprom.WriteBuf(tag, 0, 5);
						#else
						eeprom_write_block(tag, (uint8_t *) 0, 5);
						#endif
						GiveAccess();
					}
					else
						if (admin)
							AddCurrentTag();
						else
							DenyAccess();
				}
				else
					eeprom_ptr += 5;
			break;

			
		case TMM_IDLE:
			if (timer.IsFlagged())
			{
				admin = false;
				RELAISOFF;
				mode = TMM_IDLE;
				LEDOFF;
			}
			break;
	}
}

void Tagmanager::ProcessTag(uint8_t *tag)
{
	if (mode == TMM_IDLE)
	{
		if (bit_is_clear(PINB, PB3))
			GiveAccess(); //Jumper is set, allow all tags
		else
		{
			eeprom_ptr = 0;
			memcpy(this->tag, tag, 5);
			mode = TMM_LOOKUP;
		}
	}
}

void Tagmanager::GiveAccess(void)
{	
	mode = TMM_IDLE;
	
	bus.OnTag(tag, true);
	
	RELAISON;
	timer.SetTime(2000);
	#ifdef LEDGREEN
	LEDGREEN;
	#endif
	#ifdef LEDON
	LEDON;
	#endif
}

void Tagmanager::DenyAccess(void)
{
	mode = TMM_IDLE;
	
	bus.OnTag(tag, false);
	timer.SetTime(500);
	#ifdef LEDRED
	LEDRED;
	#endif
	#ifdef LEDON
	LEDON;
	#endif
}

void Tagmanager::AddCurrentTag(void)
{
	mode = TMM_IDLE;
	admin = false;
	#ifdef USEEXTEEPROM
	i2ceeprom.WriteBuf(tag, eeprom_ptr, 5);
	#else
	eeprom_write_block(tag, (uint8_t *) eeprom_ptr, 5);
	#endif
}
