/*-----------------------------------------------------------------------------
FILENAME: timer.cpp
AUTHOR: S. Seegel, post@seegel-systeme.de
DATE: 22.7.2008
VERSION: 1.0
LICENSE: GPL v3
DESCRIPTION: Implementation of Timer class
-----------------------------------------------------------------------------*/

#include "timer.h"
#include <avr/interrupt.h>

static Timer *timer_vector[MAX_NUM_TIMERS];
static uint8_t timer_vector_cnt;

Timer::Timer(void)
{
	flag = 0;
	timer_vector[timer_vector_cnt++] = this;
}

void Timer::OnIsr(void)
{
	if (cnt)
	{
		if (--cnt == 0)
			flag = 1;
	}
}

bool Timer::IsFlagged(void)
{
	if (flag)
	{
		flag = false;
		return true;
	}
	
	return false;
}

void Timer::SetTime(uint16_t time)
{
	TIMSK &= ~(1<<OCIE2);
	cnt = time;
	
	flag = 0;
		
	TIMSK |= 1<<OCIE2;
}

ISR(TIMER2_COMP_vect)
{
	uint8_t i;
	
	for (i=0; i<timer_vector_cnt; i++)
	{
		(*timer_vector[i]).OnIsr();
	}
}
